//
//   File : kvi_menubar.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_menubar.cpp)
//   Last major modification : Tue Jul 6 1999 14:46:33 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//


#define _KVI_DEBUG_CLASS_NAME_ "KviMenuBar"
#include "kvi_debug.h"

#include "kvi_menubar.h"
#include "kvi_frame.h"
#include "kvi_mdi.h"
#include "kvi_app.h"
#include "kvi_locale.h"

#include "kvi_toolbar.h"
#include "kvi_systray.h"
#include "kvi_taskbar.h"
#include "kvi_statusbar.h"
#include "kvi_utoolbar.h"

#include <qpopupmenu.h>
#include <qpainter.h>

extern KviUserToolBarTemplate * g_pUserToolBarTemplate;

//============ KviMenuBar ============//

KviMenuBar::KviMenuBar(KviFrame *par)
:QMenuBar(par,"menu_bar")
{
	m_pFrm = par;

	//Create the menu
	m_pMainPopup = new QPopupMenu(this,"main_popup");
	m_pMainPopup->insertItem(__tr("&New server window"),g_pApp,SLOT(createNewFrame()));
	m_pMainPopup->insertItem(__tr("&Close this server window"),m_pFrm,SLOT(close()));
	m_pMainPopup->insertSeparator();
	m_pMainPopup->insertItem(__tr("Connect/Disconnect"),m_pFrm,SLOT(slot_connectToServer()));
	m_pMainPopup->insertItem(__tr("Recent servers"),m_pFrm->m_pRecentServersPopup);
	m_pMainPopup->insertSeparator();
	m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Show dock widget"),g_pApp,SLOT(slot_createWMDockWidget()));
	m_pMainPopup->insertSeparator();
	m_pMainPopup->insertItem(__tr("&Quit KVIrc"),g_pApp,SLOT(slot_safeQuit()));

	QPopupMenu * opts = new QPopupMenu(this,"options_popup");
	opts->insertItem(__tr("&Nickname , username..."),g_pApp,SLOT(slot_doUserOptionsDialog()));
	opts->insertItem(__tr("&Servers and proxy hosts"),g_pApp,SLOT(slot_doServerOptionsDialog()));
//	opts->insertItem(__tr("&User interface settings"),g_pApp,SLOT(slot_doGuiOptionsDialog()));
	opts->insertItem(__tr("&General options"),g_pApp,SLOT(slot_generalOptionsDialog()));
	opts->insertItem(__tr("&Registered users database"),g_pApp,SLOT(slot_doRegUsersDialog()));
//	opts->insertItem(__tr("&Dcc options"),g_pApp,SLOT(slot_doDccOptionsDialog()));
	opts->insertSeparator();
	opts->insertItem(__tr("&Save options now"),g_pApp,SLOT(slot_saveSettings()));

	m_pToolsPopup = new QPopupMenu(this,"tools_popup");
	connect(m_pToolsPopup,SIGNAL(aboutToShow()),this,SLOT(fillToolsPopup()));

	m_pOnlinePopup = new QPopupMenu(this,"online_popup");
	m_pOnlinePopup->insertItem(__tr("Recent &channels"),m_pFrm->m_pRecentChannelsPopup);
	m_pOnlinePopup->insertItem(__tr("Recent &nicknames"),m_pFrm->m_pRecentNicknamesPopup);
	m_pOnlinePopup->insertSeparator();
	m_pOnlinePopup->insertItem(__tr("Request &links from server"),m_pFrm,SLOT(slot_requestLinks()));

	QPopupMenu * help = new QPopupMenu(this,"help_popup");
	help->insertItem(__tr("&Help window"),m_pFrm,SLOT(toggleHelpWindow()));
	help->insertItem(__tr("&What is this mode"),m_pFrm,SLOT(slot_whatIsThisRequest()));
	help->insertSeparator();
	help->insertItem(__tr("&Tip of the Day"),g_pApp,SLOT(slot_doTipOfTheDay()));
	help->insertSeparator();
	help->insertItem(__tr("&About KVirc"),g_pApp,SLOT(slot_doAboutDialog()));

	insertItem(__tr("&KVIrc"),m_pMainPopup);
	insertItem(__tr("&Options"),opts);
	insertItem(__tr("&Tools"),m_pToolsPopup);
	m_iOnlinePopupId = insertItem(__tr("On-&Line"),m_pOnlinePopup);
	setItemEnabled(m_iOnlinePopupId,false);
	insertItem(__tr("&Window"),m_pFrm->m_pMdi->getWindowMenu());
	insertItem(__tr("&Help"),help);

	setBackgroundMode(QWidget::PaletteBackground);
}

//============ ~KviMenuBar ============//

KviMenuBar::~KviMenuBar()
{
//	__debug("~KviMenuBar");
//	delete m_pOnlinePopup;
}

void KviMenuBar::enableOnLinePopup(bool bEnable)
{
	setItemEnabled(m_iOnlinePopupId,bEnable);
}

void KviMenuBar::wmDockWidgetStateChanged(bool bIsVisible)
{
	m_pMainPopup->removeItem(m_iDockWidgetId);
	if(bIsVisible){
		m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Hide dock widget"),g_pApp,SLOT(slot_destroyWMDockWidget()),0,m_iDockWidgetId,6);
	} else {
		m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Show dock widget"),g_pApp,SLOT(slot_createWMDockWidget()),0,m_iDockWidgetId,6);
	}
}

void KviMenuBar::setBackgroundPixmap(const QPixmap &pm)
{
	QMenuBar::setBackgroundPixmap(pm);
	setPalette(palette());
}

void KviMenuBar::setPalette(const QPalette &pal)
{
	if(backgroundPixmap()){
		if(!backgroundPixmap()->isNull()){
			QPalette p=pal;
			QColorGroup g=p.normal();
			g.setBrush(QColorGroup::Button,QBrush(NoBrush));
			p.setNormal(g);
			g = p.disabled();
			g.setBrush(QColorGroup::Button,QBrush(NoBrush));
			p.setDisabled(g);			
			QMenuBar::setPalette(p);
			return;
		}
	}
	QMenuBar::setPalette(pal);
}

void KviMenuBar::drawContents(QPainter *p)
{
	// It is horrible , I know it...but there is no other human way.
	const QPixmap * pix = backgroundPixmap();
	if(pix){
		if(!pix->isNull()){
			p->drawTiledPixmap(0,0,width(),height(),*pix,0,0);
			QMenuBar::drawContents(p);
			return;
		}
	}
	QMenuBar::drawContents(p);
}

void KviMenuBar::fillToolsPopup()
{
	m_pToolsPopup->clear();
	int id = m_pToolsPopup->insertItem(__tr("&Script center"),m_pFrm,SLOT(slot_showScriptCenter()));
	m_pToolsPopup->setItemEnabled(id,!g_pApp->scriptCenterDialogOpen());
	id = m_pToolsPopup->insertItem(__tr("&Directory browser"),m_pFrm,SLOT(toggleDirBrowser()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->dirBrowserOpen());
	m_pToolsPopup->insertItem(__tr("Channel &list window"),m_pFrm,SLOT(activateListWindow()));

	m_pToolsPopup->insertSeparator();

	m_pToolsPopup->insertItem(__tr("Save &current script"),g_pApp,SLOT(slot_doScriptWizard()));
	m_pToolsPopup->insertItem(__tr("&Load a script"),g_pApp,SLOT(slot_doLoadScriptDialog()));

	m_pToolsPopup->insertSeparator();

	id = m_pToolsPopup->insertItem(__tr("Status&Bar"),m_pFrm,SLOT(slot_toggleStatusBar()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pStatusBar->isVisible());
	id = m_pToolsPopup->insertItem(__tr("TaskB&ar"),m_pFrm,SLOT(slot_toggleTaskBar()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pTaskBar->isVisible());
	id = m_pToolsPopup->insertItem(__tr("ToolBa&r"),m_pFrm,SLOT(slot_toggleToolBar()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pToolBar->isVisible());
	id = m_pToolsPopup->insertItem(__tr("MdiToolBa&r"),m_pFrm,SLOT(slot_toggleMdiToolBar()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pMdiToolBar->isVisible());
	id = m_pToolsPopup->insertItem(__tr("S&ystemTrayBar"),m_pFrm,SLOT(slot_toggleSysTrayBar()));
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pSysTrayBar->isVisible());
	id = m_pToolsPopup->insertItem(__tr("&UserToolBar"),m_pFrm,SLOT(slot_toggleUserToolBar()));
	m_pToolsPopup->setItemEnabled(id,!g_pUserToolBarTemplate->isEmpty());
	m_pToolsPopup->setItemChecked(id,m_pFrm->m_pUserToolBar->isVisible());

	m_pToolsPopup->insertSeparator();

	m_pToolsPopup->insertItem(__tr("Arrange toolbars (normal)"),m_pFrm,SLOT(slot_lineUpToolBarsNormal()));
	m_pToolsPopup->insertItem(__tr("Arrange toolbars (compact)"),m_pFrm,SLOT(slot_lineUpToolBarsCompact()));
}


#include "m_kvi_menubar.moc"
